// ---------------------------------------------------------------------------
// 
//   Lily Engine alpha
// 
//   Copyright (c) 2010 Clifford Jolly
//     http://cliffjolly.com
//     cliff@cliffjolly.com
// 
// ---------------------------------------------------------------------------
// 
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//  
//   This program is distributed in the hope that it will be useful, but
//   WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
//   General Public License for more details.
// 
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
//   USA
// -------------------------- END HEADER -------------------------------------

#include <map>
#include <string>
using namespace std;

#include "../filesystem/filesystem.h"

#include "malfont.h"
#include "fontmanager.h"

FontManager mainFontManager;

FontManager::FontManager(void) {
}

FontManager::~FontManager(void) {
	freeEverything();
}

MalFont *FontManager::getFont(const std::string &name) {
	string fixedName = FileSystem::fixFileName(name);
	
	if(!fonts[fixedName]) {
		MalFont *newFont = new MalFont();
		newFont->loadFont(fixedName);
		fonts[fixedName] = newFont;
	}
	
	return fonts[fixedName];
}
	
void FontManager::freeEverything(void) {
	
	for(map<string, MalFont*>::iterator i = fonts.begin(); i != fonts.end(); i++) {
		delete (*i).second;
	}
	
	fonts.clear();
}

FontManager *FontManager::get(void) {
	static FontManager mainFontManager;
	
	return &mainFontManager;
}

std::string FontManager::getNameForFont(const MalFont *font) {
	if(!font) return "";
	
	for(map<string, MalFont*>::iterator i = fonts.begin(); i != fonts.end(); i++) {
		if((*i).second == font) {
			return (*i).first;
		}
	}
	
	// Font existing outside of FontManager?
	return "";
}
