// ---------------------------------------------------------------------------
// 
//   Lily Engine alpha
// 
//   Copyright (c) 2010 Clifford Jolly
//     http://cliffjolly.com
//     cliff@cliffjolly.com
// 
// ---------------------------------------------------------------------------
// 
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//  
//   This program is distributed in the hope that it will be useful, but
//   WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
//   General Public License for more details.
// 
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
//   USA
// -------------------------- END HEADER -------------------------------------

#ifndef MALTHREAD_H
#define MALTHREAD_H

#include <boost/function.hpp>

namespace Threads {

#ifdef WIN32
#include <windows.h>
    typedef DWORD ThreadId;
    typedef HANDLE ThreadType;
#else
    typedef pthread_t ThreadId;
    typedef pthread_t ThreadType;
#endif

    /// Get the current thread's ID.
    ThreadId getMyId(void);

    class Mutex {

        public:

            Mutex(const Mutex &m);
            Mutex(void);
            ~Mutex(void);

            void lock(void);
            void unlock(void);

        private:

            struct MutexPrivate;
            MutexPrivate *mutexPrivate;
    };

    class Thread {

        public:
        
            Thread(void);
            Thread(const Thread &t);
            Thread(boost::function<void()> func);
            Thread(void (*bareFunc)(void*), void *bareData);
            ~Thread(void);

            void join(void);
            ThreadId getId(void);

        private:

            void finishInit(void);

            bool done;

            struct ThreadPrivate;
            ThreadPrivate *threadPrivate;
            ThreadType systemThread;

            // Entry point for threads.
#ifdef WIN32
            friend unsigned int __stdcall threadStarter(void *data);
#else
            friend void *threadStarter(void *data);
#endif

    };

};

#endif
