// ---------------------------------------------------------------------------
// 
//   Lily Engine alpha
// 
//   Copyright (c) 2010 Clifford Jolly
//     http://cliffjolly.com
//     cliff@cliffjolly.com
// 
// ---------------------------------------------------------------------------
// 
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//  
//   This program is distributed in the hope that it will be useful, but
//   WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
//   General Public License for more details.
// 
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
//   USA
// -------------------------- END HEADER -------------------------------------

#pragma once

#include <string>

#include "../3rdparty/tinyxml/tinyxml.h"

/** @brief Save a boolean value to an XML element.
 * @param xml The TiXmlElement.
 * @param name The attribute name.
 * @param value The value to save.
 * */
inline void setBoolAttributeInXml(TiXmlElement *xml, const std::string &name, bool value) {
	// I'm not going to set false values because the XML files are
	// getting huge with stuff that doesn't matter. Values that aren't
	// there will just come in as a false, anyway.
	if(!value) return;
	
	xml->SetAttribute(name.c_str(), (int)value);
}

/** @brief Load a boolean attribute from an XML element.
 * @param xml The TiXmlElement.
 * @param name The attribute name.
 * @return The value.
 * */
inline bool getBoolAttributeFromXml(TiXmlElement *xml, const std::string &name) {
	int val = 0;
	xml->QueryIntAttribute(name.c_str(), &val);
	return (bool)val;
}

/// Just load an XML file from a disk. Uses FileSystem, so hits
/// archives too.
TiXmlDocument *loadXml(const std::string &fileName);
