// ---------------------------------------------------------------------------
// 
//   Lily Engine alpha
// 
//   Copyright (c) 2010 Clifford Jolly
//     http://cliffjolly.com
//     cliff@cliffjolly.com
// 
// ---------------------------------------------------------------------------
// 
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//  
//   This program is distributed in the hope that it will be useful, but
//   WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
//   General Public License for more details.
// 
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
//   USA
// -------------------------- END HEADER -------------------------------------

#ifndef WIDGETSELECTCHILD_H
#define WIDGETSELECTCHILD_H

#include "widgetmultiparent.h"

/// WidgetMultiParent that only shows the currently focused child. Also
/// can be connected to WidgetButtons (preferably as radio buttons) to
/// allow the user to select and individual child.
LUA_WRAP class WidgetSelectChild : public WidgetMultiParent {
	public:
		
		// Unique stuff.
		
		/// Map a radio button to a child so we'll switch focus to that
		/// child when the radio button is active.
		LUA_WRAP void mapRadioToChild(Widget *radio, Widget *child);
		
		/// Clear all the radio/child mappings.
		LUA_WRAP void clearRadioToChildMaps(void);
		
		// Constructor/destructor.
		WidgetSelectChild(void);
		virtual ~WidgetSelectChild(void);
		
		// Inherited stuff.
		void draw(void);
		float getMinHeight(void);
		float getMinWidth(void);
		void setWidth(float newWidth);
		void setHeight(float newHeight);
		bool handleEvent(const WidgetEvent *event);
		void switchFocus(int newFocus);
		
	private:
	
		std::map<std::string, int> radioToChildMap;
};

#endif
