// ---------------------------------------------------------------------------
// 
//   Lily Engine alpha
// 
//   Copyright (c) 2010 Clifford Jolly
//     http://cliffjolly.com
//     cliff@cliffjolly.com
// 
// ---------------------------------------------------------------------------
// 
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//  
//   This program is distributed in the hope that it will be useful, but
//   WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
//   General Public License for more details.
// 
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
//   USA
// -------------------------- END HEADER -------------------------------------

#ifndef WIDGETSINGLEPARENT_H
#define WIDGETSINGLEPARENT_H

#include "widget.h"

/// WidgetSingleParent is an abstract class with some of the
/// functionality common to widgets that contain a single child (which
/// could very well be a widget that contains more or multiple
/// children). Classes inheriting from WidgetSingleParent should
/// implement the draw() function and the handleEvent() function.
LUA_WRAP class WidgetSingleParent : public Widget {
	
	public:
	
		// Stuff unique to this.
		
		/// Set the widget's child.
		LUA_WRAP void setChild(Widget *child);
		
		/// Get the widget's child.
		LUA_WRAP Widget *getChild(void);

		/// Remove a child without deleting it.
		LUA_WRAP Widget *disconnectChild(void);

		// Constructors/destructor
		WidgetSingleParent(void);
		WidgetSingleParent(TiXmlElement *xml);
		virtual ~WidgetSingleParent(void);
	
		// Stuff inherited from Widget.
		bool needsMouseEvents(void);
		bool handleEvent(const WidgetEvent *event);
		void draw(void);
		bool isSizeDirty(void);
		Widget *findChildByName(const std::string &name);
		
	protected:
		
		/// The child widget.
		Widget *child;
	
};

void widgetSingleParentInitLua(lua_State *L);

#endif
